import { Component, inject, OnDestroy, OnInit } from '@angular/core';
import { ActionListComponent } from "../action-list/action-list.component";
import { MatSelectModule } from '@angular/material/select';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatInputModule } from '@angular/material/input';
import { MatIconModule } from '@angular/material/icon';
import { HermesClientService } from '../../hermes-client.service';
import RedeemableAction from '../../shared/models/redeemable-action';
import { FormControl, ReactiveFormsModule } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import RedeemableActionService from '../../shared/services/redeemable-action.service';
import { MatButtonModule } from '@angular/material/button';
import { containsLettersInOrder } from '../../shared/utils/string-compare';
import { Subscription } from 'rxjs';

interface IActionFilter {
  name: string
  filter: (action: any) => boolean
}

@Component({
  selector: 'actions',
  imports: [
    ActionListComponent,
    MatButtonModule,
    MatFormFieldModule,
    MatIconModule,
    MatInputModule,
    MatSelectModule,
    ReactiveFormsModule,
    ReactiveFormsModule,
  ],
  templateUrl: './actions.component.html',
  styleUrl: './actions.component.scss'
})
export class ActionsComponent implements OnInit, OnDestroy {
  filters: IActionFilter[] = [
    { name: 'All', filter: _ => true },
    { name: 'Local File', filter: data => data.type.includes('_FILE') },
    { name: 'Nightbot', filter: data => data.type.includes('NIGHTBOT_') },
    { name: 'OBS', filter: data => data.type.includes('OBS_') },
    { name: 'Sleep', filter: data => data.type == "SLEEP" },
    { name: 'TTS', filter: data => data.type.includes('TTS') },
    { name: 'Veadotube', filter: data => data.type.includes('VEADOTUBE') },
  ];

  private readonly client = inject(HermesClientService);
  private readonly redeemableActionService = inject(RedeemableActionService);
  private readonly route = inject(ActivatedRoute);

  private readonly subscriptions: (Subscription | undefined)[] = [];

  filter = this.filters[0];
  searchControl = new FormControl<string>('');
  _actions: RedeemableAction[] = [];


  ngOnInit(): void {
    this.route.data.subscribe(data => this._actions = data['redeemableActions'] || []);

    this.subscriptions.push(this.redeemableActionService.delete$?.subscribe(_ => this.redeemableActionService.fetch().subscribe(a => this._actions = a)));

    this.client.fetchRedeemableActions();
  }

  ngOnDestroy(): void {
    this.subscriptions.filter(s => s).forEach(s => s?.unsubscribe());
  }

  get actions(): RedeemableAction[] {
    const searchLower = this.searchControl.value!.toLowerCase();
    return this._actions.filter(this.filter.filter)
      .filter((action) => containsLettersInOrder(action.name.toLowerCase(), searchLower));
  }
}
