import { HttpClient } from '@angular/common/http';
import { Component, Inject, inject } from '@angular/core';
import { FormControl, FormGroup, ReactiveFormsModule, Validators } from '@angular/forms';
import { MatButtonModule } from '@angular/material/button';
import { MatCardModule } from '@angular/material/card';
import { MAT_DIALOG_DATA, MatDialogRef } from '@angular/material/dialog';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatIconModule } from '@angular/material/icon';
import { MatInputModule } from '@angular/material/input';
import { MatSelectModule } from '@angular/material/select';
import { DOCUMENT } from '@angular/common';
import { environment } from '../../../environments/environment';

@Component({
  selector: 'connection-item-edit',
  imports: [
    MatButtonModule,
    MatCardModule,
    MatIconModule,
    MatFormFieldModule,
    MatInputModule,
    MatSelectModule,
    ReactiveFormsModule,
  ],
  templateUrl: './connection-item-edit.component.html',
  styleUrl: './connection-item-edit.component.scss'
})
export class ConnectionItemEditComponent {
  private readonly http = inject(HttpClient);
  private readonly data = inject<{ name: string }>(MAT_DIALOG_DATA);
  readonly dialogRef = inject(MatDialogRef<ConnectionItemEditComponent>);

  readonly nameControl = new FormControl<string>('', [Validators.required]);
  readonly clientIdControl = new FormControl<string>('', [Validators.required]);
  readonly typeControl = new FormControl<string>('', [Validators.required]);
  readonly form = new FormGroup({
    name: this.nameControl,
    clientId: this.clientIdControl,
    type: this.typeControl,
  });


  responseError: string | undefined;
  waitForResponse = false;

  constructor(@Inject(DOCUMENT) private document: Document) { }


  ngOnInit(): void {
    this.nameControl.setValue(this.data.name);
  }

  submit(): void {
    if (!this.form.dirty || this.form.invalid || this.waitForResponse) {
      return;
    }

    this.http.post(environment.API_HOST + '/auth/connections', {
      name: this.nameControl.value,
      type: this.typeControl.value,
      client_id: this.clientIdControl.value,
      grant_type: 'bearer',
    },
      {
        headers: {
          'Authorization': 'Bearer ' + localStorage.getItem('jwt')
        }
      }).subscribe(async (d: any) => this.document.location.href = d.data);
  }
}
