import { Component, EventEmitter, inject, input, Input, OnInit, Output } from '@angular/core';
import { FormControl, ReactiveFormsModule, Validators } from '@angular/forms';
import { MatAutocompleteModule } from '@angular/material/autocomplete';
import { MatFormFieldModule } from '@angular/material/form-field';
import TwitchRedemption from '../../shared/models/twitch-redemption';
import { MatInputModule } from '@angular/material/input';
import { ActivatedRoute } from '@angular/router';

@Component({
  selector: 'twitch-redemption-dropdown',
  imports: [MatAutocompleteModule, MatFormFieldModule, MatInputModule, ReactiveFormsModule],
  templateUrl: './twitch-redemption-dropdown.component.html',
  styleUrl: './twitch-redemption-dropdown.component.scss'
})
export class TwitchRedemptionDropdownComponent implements OnInit {
  @Input() formControl = new FormControl<TwitchRedemption | string | undefined>(undefined);
  @Input() errorMessages: { [errorKey: string]: string } = {};

  @Input() search: boolean = false;
  @Input() twitchRedemptions: TwitchRedemption[] = [];
  @Input() twitchRedemptionId: string | undefined;
  @Output() readonly twitchRedemptionIdChange = new EventEmitter<string>();

  private readonly route = inject(ActivatedRoute);

  errorMessageKeys: string[] = [];

  constructor() {
    this.route.data.subscribe(data => {
      if (!data['twitchRedemptions'])
        return;

      this.twitchRedemptions = data['twitchRedemptions'];
    });
  }

  ngOnInit(): void {
    this.errorMessageKeys = Object.keys(this.errorMessages);

    if (!this.twitchRedemptionId || !this.twitchRedemptions)
      return;

    const redemption = this.twitchRedemptions.find(r => r.id == this.twitchRedemptionId);
    this.formControl.setValue(redemption);
  }

  get filteredRedemptions() {
    const value = this.formControl.value;
    if (typeof (value) == 'string') {
      return this.twitchRedemptions.filter(r => r.title.toLowerCase().includes(value.toLowerCase()));
    }
    return this.twitchRedemptions;
  }

  select(event: TwitchRedemption) {
    this.twitchRedemptionIdChange.emit(event.id);
  }

  input() {
    if (this.search && typeof this.formControl.value == 'string') {
      this.twitchRedemptionIdChange.emit(this.formControl.value);
    }
  }

  blur() {
    if (!this.search && typeof this.formControl.value == 'string') {
      const name = this.formControl.value;
      const nameLower = name.toLowerCase();
      let newValue: TwitchRedemption | undefined = undefined;
      const insenstiveActions = this.filteredRedemptions.filter(a => a.title.toLowerCase() == nameLower);
      if (insenstiveActions.length > 1) {
        const sensitiveAction = insenstiveActions.find(a => a.title == name);
        newValue = sensitiveAction ?? undefined;
      } else if (insenstiveActions.length == 1) {
        newValue = insenstiveActions[0];
      }
      if (newValue && newValue.id != this.formControl.value) {
        this.formControl.setValue(newValue);
        this.twitchRedemptionIdChange.emit(newValue.id);
      } else if (!newValue)
        this.twitchRedemptionIdChange.emit(undefined);
    }
  }

  displayFn(value: any) {
    return value?.title;
  }
}
