import { inject, Injectable } from '@angular/core';
import { HermesClientService } from '../../hermes-client.service';
import { first, map, Observable, of } from 'rxjs';
import EventService from './EventService';
import { Filter } from '../models/filter';

@Injectable({
  providedIn: 'root'
})
export default class TtsFilterService {
  private readonly client = inject(HermesClientService);
  private readonly events = inject(EventService);
  private data: Filter[] = []
  private loaded = false;
  create$: Observable<any> | undefined;
  update$: Observable<any> | undefined;
  delete$: Observable<any> | undefined;

  constructor() {
    this.create$ = this.client.filterByRequestType('create_tts_filter');
    this.update$ = this.client.filterByRequestType('update_tts_filter');
    this.delete$ = this.client.filterByRequestType('delete_tts_filter');

    this.create$?.subscribe(d => this.data.push(d.data));
    this.update$?.subscribe(d => {
      const filter = this.data.find(r => r.id == d.data.id);
      if (filter) {
        filter.search = d.data.action_name;
        filter.replace = d.data.redemption_id;
        filter.flag = d.data.order;
      }
    });
    this.delete$?.subscribe(d => this.data = this.data.filter(r => r.id != d.request.data.id));

    this.events.listen('tts_logoff', () => {
      this.data = [];
      this.loaded = false;
    });
  }


  fetch() {
    if (this.loaded) {
      return of(this.data);
    }

    const $ = this.client.first(d => d.op == 4 && d.d.request.type == 'get_tts_word_filters')!.pipe(map(d => d.d.data));
    $.subscribe(d => {
      this.data = d;
      this.loaded = true;
    });
    this.client.fetchFilters();
    return $;
  }
}
