import { isPlatformBrowser } from '@angular/common';
import { HttpClient } from '@angular/common/http';
import { Component, inject, Inject, OnDestroy, OnInit, PLATFORM_ID } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { ApiAuthenticationService } from '../shared/services/api/api-authentication.service';
import { environment } from '../../environments/environment';
import { Subscription } from 'rxjs';

@Component({
  selector: 'app-twitch-auth-callback',
  standalone: true,
  imports: [],
  templateUrl: './twitch-auth-callback.component.html',
  styleUrl: './twitch-auth-callback.component.scss'
})
export class TwitchAuthCallbackComponent implements OnInit, OnDestroy {
  private readonly auth = inject(ApiAuthenticationService);
  private readonly http = inject(HttpClient);
  private readonly route = inject(ActivatedRoute);
  private readonly router = inject(Router);
  private readonly subscriptions: (Subscription | null)[] = [];

  constructor(@Inject(PLATFORM_ID) private platformId: Object) { }

  async ngOnInit(): Promise<any> {
    if (!isPlatformBrowser(this.platformId)) {
      return;
    }

    if (!this.auth.isAuthenticated() && localStorage.getItem('jwt')) {
      localStorage.removeItem('jwt');
    }

    if (this.auth.isAuthenticated() || localStorage.getItem('jwt')) {
      await this.router.navigate(['policies']);
      return;
    }

    const code = this.route.snapshot.queryParamMap.get('code');
    const scope = this.route.snapshot.queryParamMap.get('scope');
    const state = this.route.snapshot.queryParamMap.get('state');

    if (!code || !scope || !state) {
      await this.router.navigate(['login'], {
        queryParams: {
          error: 'callback_issue'
        }
      });
      return;
    }

    this.http.post(environment.API_HOST + '/auth/twitch/callback', { code, scope, state })
      .subscribe({
        next: async (response: any) => {
          this.auth.update(response.token);
        },
        error: async () => await this.router.navigate(['login'], {
          queryParams: {
            error: 'callback_issue_twitch'
          }
        }),
      });
  }

  ngOnDestroy(): void {
    for (let subscription of this.subscriptions) {
      if (subscription)
        subscription.unsubscribe();
    }
  }
}
